#include <Arduino.h>
#include <U8x8lib.h> 

#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif

U8X8_SH1106_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE); 	

// Rotary encoders inputs
#define S1A 4
#define S2A 3
#define KEYA 2
#define S1B 7
#define S2B 6
#define KEYB 5

// Rotary encoders variables
int counterA = 0;
int currentStateA;
int lastStateA;
String currentDirA ="";
int xPosA = 2;
int yPosA = 0;

int counterB = 0;
int currentStateB;
int lastStateB;
String currentDirB ="";
unsigned long lastButtonPress = 0;
int xPosB = 2;
int yPosB = 12;

void setup() {
  u8x8.begin();
  u8x8.setPowerSave(0);
  u8x8.setFont(u8x8_font_px437wyse700b_2x2_f); // Bigger font type
  u8x8.setFlipMode(1); // Flip screen 180 degrees because screen is installed upside down
  u8x8.setInverseFont(0); // Displays white text

  Serial.begin(9600);

  // Set encoder pins as inputs
	pinMode(S1A,INPUT);
	pinMode(S2A,INPUT);
	pinMode(KEYA, INPUT_PULLUP);

  pinMode(S1B,INPUT);
	pinMode(S2B,INPUT);
	pinMode(KEYB, INPUT_PULLUP);

	// Read the initial state of S1
	lastStateA = digitalRead(S1A);
  lastStateB = digitalRead(S1B);
}

void loop() {
  // Read the current state of S1
	currentStateA = digitalRead(S1A);

  if (counterA > 10 || counterA < -10)
  {
    counterA = 0; // If rotary encoder is turned more than 10 times reset the counter to 0
  }
  
	// If last and current state of S1 are different, then pulse occurred
	// React to only 1 state change to avoid double count
	if (currentStateA != lastStateA  && currentStateA == 1)
  {
		// If the S2 state is different than the S1 state then
		// the encoder is rotating CCW so decrement
		if (digitalRead(S2A) != currentStateA) {
			counterA --;
			currentDirA ="CCW";
		} else {
			// Encoder is rotating CW so increment
			counterA ++;
			currentDirA ="CW";
		}

    // Draw eyes
    if (counterA == 0 || counterA == 11 || counterA == -11) {
      u8x8.draw2x2String(xPosA, yPosA, "U U");
    }
    if (counterA == 1 || counterA == -10) {
      u8x8.draw2x2String(xPosA, yPosA, "| |");
    }
    if (counterA == 2 || counterA == -9) {
      u8x8.draw2x2String(xPosA, yPosA,"^ ^");
    }
    if (counterA == 3 || counterA == -8) {
      u8x8.draw2x2String(xPosA, yPosA,"O O");
    }
    if (counterA == 4 || counterA == -7) {
      u8x8.draw2x2String(xPosA, yPosA,"> <");
    }
    if (counterA == 5 || counterA == -6) {
      u8x8.draw2x2String(xPosA, yPosA,"n n"); 
    }
    if (counterA == 6 || counterA == -5) {
      u8x8.draw2x2String(xPosA, yPosA,"@ @");
    }
    if (counterA == 7 || counterA == -4) {
      u8x8.draw2x2String(xPosA, yPosA,"- -");
    }
    if (counterA == 8 || counterA == -3) {
      u8x8.draw2x2String(xPosA, yPosA,"T T");
    }
    if (counterA == 9 || counterA == -2) {
      u8x8.draw2x2String(xPosA, yPosA,"X X");
    }
    if (counterA == 10 || counterA == -1) {
      u8x8.draw2x2String(xPosA, yPosA,"o O");
    }

		Serial.print("DirectionA: ");
		Serial.print(currentDirA);
		Serial.print(" | CounterA: ");
		Serial.println(counterA);
	}

	// Remember last S1 state
	lastStateA = currentStateA;

  // Read the current state of S1
	currentStateB = digitalRead(S1B);

  if (counterB > 10 || counterB < -10)
  {
    counterB = 0; // If rotary encoder is turned more than 10 times reset the counter to 0
  }

	// If last and current state of S1 are different, then pulse occurred
	// React to only 1 state change to avoid double count
	if (currentStateB != lastStateB  && currentStateB == 1)
  {
		// If the S2 state is different than the S1 state then
		// the encoder is rotating CCW so decrement
		if (digitalRead(S2B) != currentStateB) {
			counterB --;
			currentDirB ="CCW";
		} else {
			// Encoder is rotating CW so increment
			counterB ++;
			currentDirB ="CW";
		}

    // Draw mouth
    if (counterB == 0 || counterB == 11 || counterB == -11) {
      u8x8.draw2x2String(xPosB, yPosB, " - ");
    }
    if (counterB == 1 || counterB == -10) {
      u8x8.draw2x2String(xPosB, yPosB, " V ");
    }
    if (counterB == 2 || counterB == -9) {
      u8x8.draw2x2String(xPosB, yPosB," n ");
    }
    if (counterB == 3 || counterB == -8) {
      u8x8.draw2x2String(xPosB, yPosB," O ");
    }
    if (counterB == 4 || counterB == -7) {
      u8x8.draw2x2String(xPosB, yPosB," > ");
    }
    if (counterB == 5 || counterB == -6) {
      u8x8.draw2x2String(xPosB, yPosB," W ");
    }
    if (counterB == 6 || counterB == -5) {
      u8x8.draw2x2String(xPosB, yPosB," U ");
    }
    if (counterB == 7 || counterB == -4) {
      u8x8.draw2x2String(xPosB, yPosB," 3 ");
    }
    if (counterB == 8 || counterB == -3) {
      u8x8.draw2x2String(xPosB, yPosB," A ");
    }
    if (counterB == 9 || counterB == -2) {
      u8x8.draw2x2String(xPosB, yPosB," X ");
    }
    if (counterB == 10 || counterB == -1) {
      u8x8.draw2x2String(xPosB, yPosB," 3 ");
    }

		Serial.print("DirectionB: ");
		Serial.print(currentDirB);
		Serial.print(" | CounterB: ");
		Serial.println(counterB);
	}

	// Remember last S1 state
	lastStateB = currentStateB;

  //rotationState(counterB, currentStateB, lastStateB, currentDirB, S1B, S2B);
  buttonState(KEYA, xPosA, yPosA);
  buttonState(KEYB, xPosB, yPosB);
}

void buttonState (int key, int x, int y)
{
	// Read the button state
	int btnState = digitalRead(key);

	// If we detect LOW signal, button is pressed
	if (btnState == LOW)
  {
		// If 50ms have passed since last LOW pulse, it means that the
		// button has been pressed, released and pressed again
		if (millis() - lastButtonPress > 50) {
      u8x8.draw2x2String(x, y,"   ");
			Serial.println("Button pressed!");
		}

		// Remember last button press event
		lastButtonPress = millis();
	}

	// Put in a slight delay to help debounce the reading
	delay(1);
}